@@ -94,7 +94,7 @@ bool parse_url(const string &url, string *protocol, string *user, string *host,
 	return true;
 }
 
-Input *create_input(const std::string &url)
+Input *create_input(const string &url)
 {
 	string protocol, user, host, port, path;
 	if (!parse_url(url, &protocol, &user, &host, &port, &path)) {
