@@ -140,7 +140,7 @@ void Server::do_work()
 		}
 		timeout_time.tv_sec -= REQUEST_READ_TIMEOUT_SEC;
 		while (!clients_ordered_by_connect_time.empty()) {
-			pair<timespec, int> &connect_time_and_fd = clients_ordered_by_connect_time.front();
+			const pair<timespec, int> &connect_time_and_fd = clients_ordered_by_connect_time.front();
 
 			// See if we have reached the end of clients to process.
 			if (is_earlier(timeout_time, connect_time_and_fd.first)) {
@@ -149,7 +149,7 @@ void Server::do_work()
 
 			// If this client doesn't exist anymore, just ignore it
 			// (it was deleted earlier).
-			std::map<int, Client>::iterator client_it = clients.find(connect_time_and_fd.second);
+			map<int, Client>::iterator client_it = clients.find(connect_time_and_fd.second);
 			if (client_it == clients.end()) {
 				clients_ordered_by_connect_time.pop();
 				continue;
@@ -286,19 +286,19 @@ void Server::add_client_from_serialized(const ClientProto &client)
 	}
 }
 
-int Server::lookup_stream_by_url(const std::string &url) const
+int Server::lookup_stream_by_url(const string &url) const
 {
-	map<string, int>::const_iterator url_it = url_map.find(url);
-	if (url_it == url_map.end()) {
+	map<string, int>::const_iterator stream_url_it = stream_url_map.find(url);
+	if (stream_url_it == stream_url_map.end()) {
 		return -1;
 	}
-	return url_it->second;
+	return stream_url_it->second;
 }
 
 int Server::add_stream(const string &url, size_t backlog_size, size_t prebuffering_bytes, Stream::Encoding encoding)
 {
 	MutexLock lock(&mutex);
-	url_map.insert(make_pair(url, streams.size()));
+	stream_url_map.insert(make_pair(url, streams.size()));
 	streams.push_back(new Stream(url, backlog_size, prebuffering_bytes, encoding));
 	return streams.size() - 1;
 }
@@ -306,7 +306,7 @@ int Server::add_stream(const string &url, size_t backlog_size, size_t prebufferi
 int Server::add_stream_from_serialized(const StreamProto &stream, int data_fd)
 {
 	MutexLock lock(&mutex);
-	url_map.insert(make_pair(stream.url(), streams.size()));
+	stream_url_map.insert(make_pair(stream.url(), streams.size()));
 	streams.push_back(new Stream(stream, data_fd));
 	return streams.size() - 1;
 }
@@ -359,6 +359,13 @@ void Server::set_pacing_rate(int stream_index, uint32_t pacing_rate)
 	streams[stream_index]->pacing_rate = pacing_rate;
 }
 
+void Server::add_gen204(const std::string &url, const std::string &allow_origin)
+{
+	MutexLock lock(&mutex);
+	assert(clients.empty());
+	ping_url_map[url] = allow_origin;
+}
+
 void Server::add_data_deferred(int stream_index, const char *data, size_t bytes, StreamStartSuitability suitable_for_stream_start)
 {
 	assert(stream_index >= 0 && stream_index < ssize_t(streams.size()));
@@ -418,22 +425,24 @@ read_request_again:
 		int error_code = parse_request(client);
 		if (error_code == 200) {
 			construct_header(client);
+		} else if (error_code == 204) {
+			construct_204(client);
 		} else {
 			construct_error(client, error_code);
 		}
 
 		// We've changed states, so fall through.
-		assert(client->state == Client::SENDING_ERROR ||
+		assert(client->state == Client::SENDING_SHORT_RESPONSE ||
 		       client->state == Client::SENDING_HEADER);
 	}
-	case Client::SENDING_ERROR:
+	case Client::SENDING_SHORT_RESPONSE:
 	case Client::SENDING_HEADER: {
-sending_header_or_error_again:
+sending_header_or_short_response_again:
 		int ret;
 		do {
 			ret = write(client->sock,
-		                    client->header_or_error.data() + client->header_or_error_bytes_sent,
-				    client->header_or_error.size() - client->header_or_error_bytes_sent);
+		                    client->header_or_short_response.data() + client->header_or_short_response_bytes_sent,
+				    client->header_or_short_response.size() - client->header_or_short_response_bytes_sent);
 		} while (ret == -1 && errno == EINTR);
 
 		if (ret == -1 && errno == EAGAIN) {
@@ -451,18 +460,18 @@ sending_header_or_error_again:
 			return;
 		}
 		
-		client->header_or_error_bytes_sent += ret;
-		assert(client->header_or_error_bytes_sent <= client->header_or_error.size());
+		client->header_or_short_response_bytes_sent += ret;
+		assert(client->header_or_short_response_bytes_sent <= client->header_or_short_response.size());
 
-		if (client->header_or_error_bytes_sent < client->header_or_error.size()) {
+		if (client->header_or_short_response_bytes_sent < client->header_or_short_response.size()) {
 			// We haven't sent all yet. Fine; go another round.
-			goto sending_header_or_error_again;
+			goto sending_header_or_short_response_again;
 		}
 
 		// We're done sending the header or error! Clear it to release some memory.
-		client->header_or_error.clear();
+		client->header_or_short_response.clear();
 
-		if (client->state == Client::SENDING_ERROR) {
+		if (client->state == Client::SENDING_SHORT_RESPONSE) {
 			// We're done sending the error, so now close.	
 			// This is postcondition #1.
 			close_client(client);
@@ -472,7 +481,7 @@ sending_header_or_error_again:
 		Stream *stream = client->stream;
 		if (client->stream_pos == size_t(-2)) {
 			// Start sending from the beginning of the backlog.
-			client->stream_pos = std::min<size_t>(
+			client->stream_pos = min<size_t>(
 			    stream->bytes_received - stream->backlog_size,
 			    0);
 			client->state = Client::SENDING_DATA;
@@ -636,6 +645,7 @@ int Server::parse_request(Client *client)
 	}
 
 	string url = request_tokens[1];
+	client->url = url;
 	if (url.find("?backlog") == url.size() - 8) {
 		client->stream_pos = -2;
 		url = url.substr(0, url.size() - 8);
@@ -643,18 +653,21 @@ int Server::parse_request(Client *client)
 		client->stream_pos = -1;
 	}
 
-	map<string, int>::const_iterator url_map_it = url_map.find(url);
-	if (url_map_it == url_map.end()) {
-		return 404;  // Not found.
+	map<string, int>::const_iterator stream_url_map_it = stream_url_map.find(url);
+	if (stream_url_map_it == stream_url_map.end()) {
+		map<string, string>::const_iterator ping_url_map_it = ping_url_map.find(url);
+		if (ping_url_map_it == ping_url_map.end()) {
+			return 404;  // Not found.
+		} else {
+			return 204;  // No error.
+		}
 	}
 
-	Stream *stream = streams[url_map_it->second];
+	Stream *stream = streams[stream_url_map_it->second];
 	if (stream->http_header.empty()) {
 		return 503;  // Service unavailable.
 	}
 
-	client->url = request_tokens[1];
-
 	client->stream = stream;
 	if (setsockopt(client->sock, SOL_SOCKET, SO_MAX_PACING_RATE, &client->stream->pacing_rate, sizeof(client->stream->pacing_rate)) == -1) {
 		if (client->stream->pacing_rate != ~0U) {
@@ -670,11 +683,11 @@ void Server::construct_header(Client *client)
 {
 	Stream *stream = client->stream;
 	if (stream->encoding == Stream::STREAM_ENCODING_RAW) {
-		client->header_or_error = stream->http_header +
+		client->header_or_short_response = stream->http_header +
 			"\r\n" +
 			stream->stream_header;
 	} else if (stream->encoding == Stream::STREAM_ENCODING_METACUBE) {
-		client->header_or_error = stream->http_header +
+		client->header_or_short_response = stream->http_header +
 			"Content-encoding: metacube\r\n" +
 			"\r\n";
 		if (!stream->stream_header.empty()) {
@@ -683,10 +696,10 @@ void Server::construct_header(Client *client)
 			hdr.size = htonl(stream->stream_header.size());
 			hdr.flags = htons(METACUBE_FLAGS_HEADER);
 			hdr.csum = htons(metacube2_compute_crc(&hdr));
-			client->header_or_error.append(
+			client->header_or_short_response.append(
 				string(reinterpret_cast<char *>(&hdr), sizeof(hdr)));
 		}
-		client->header_or_error.append(stream->stream_header);
+		client->header_or_short_response.append(stream->stream_header);
 	} else {
 		assert(false);
 	}
@@ -709,10 +722,42 @@ void Server::construct_error(Client *client, int error_code)
 	char error[256];
 	snprintf(error, 256, "HTTP/1.0 %d Error\r\nContent-type: text/plain\r\n\r\nSomething went wrong. Sorry.\r\n",
 		error_code);
-	client->header_or_error = error;
+	client->header_or_short_response = error;
 
 	// Switch states.
-	client->state = Client::SENDING_ERROR;
+	client->state = Client::SENDING_SHORT_RESPONSE;
+
+	epoll_event ev;
+	ev.events = EPOLLOUT | EPOLLET | EPOLLRDHUP;
+	ev.data.u64 = reinterpret_cast<uint64_t>(client);
+
+	if (epoll_ctl(epoll_fd, EPOLL_CTL_MOD, client->sock, &ev) == -1) {
+		log_perror("epoll_ctl(EPOLL_CTL_MOD)");
+		exit(1);
+	}
+}
+
+void Server::construct_204(Client *client)
+{
+	map<string, string>::const_iterator ping_url_map_it = ping_url_map.find(client->url);
+	assert(ping_url_map_it != ping_url_map.end());
+
+	if (ping_url_map_it->second.empty()) {
+		client->header_or_short_response =
+			"HTTP/1.0 204 No Content\r\n"
+			"\r\n";
+	} else {
+		char response[256];
+		snprintf(response, 256,
+		         "HTTP/1.0 204 No Content\r\n"
+		         "Access-Control-Allow-Origin: %s\r\n"
+			 "\r\n",
+			 ping_url_map_it->second.c_str());
+		client->header_or_short_response = response;
+	}
+
+	// Switch states.
+	client->state = Client::SENDING_SHORT_RESPONSE;
 
 	epoll_event ev;
 	ev.events = EPOLLOUT | EPOLLET | EPOLLRDHUP;
