@@ -8,6 +8,7 @@
 #include <stdint.h>
 #include <sys/types.h>
 #include <sys/uio.h>
+#include <deque>
 #include <string>
 #include <vector>
 
@@ -23,7 +24,7 @@ struct Stream {
 	// Must be in sync with StreamConfig::Encoding.
 	enum Encoding { STREAM_ENCODING_RAW = 0, STREAM_ENCODING_METACUBE };
 
-	Stream(const std::string &stream_id, size_t backlog_size, Encoding encoding);
+	Stream(const std::string &stream_id, size_t backlog_size, size_t prebuffering_bytes, Encoding encoding);
 	~Stream();
 
 	// Serialization/deserialization.
@@ -63,13 +64,21 @@ struct Stream {
 	// How many bytes <data_fd> can hold (the buffer size).
 	size_t backlog_size;
 
+	// How many bytes we need to have in the backlog before we start
+	// sending (in practice, we will then send all of them at once,
+	// and then start sending at the normal rate thereafter).
+	// This is basically to force a buffer on the client, which can help
+	// if the client expects us to be able to fill up the buffer much
+	// faster than realtime (ie., it expects a static file).
+	size_t prebuffering_bytes;
+
 	// How many bytes this stream have received. Can very well be larger
 	// than <backlog_size>, since the buffer wraps.
 	size_t bytes_received;
 
-	// The last point in the stream that is suitable to start new clients at
-	// (after having sent the header). -1 if no such point exists yet.
-	ssize_t last_suitable_starting_point;
+	// A list of points in the stream that is suitable to start new clients at
+	// (after having sent the header). Empty if no such point exists yet.
+	std::deque<size_t> suitable_starting_points;
 	
 	// Clients that are in SENDING_DATA, but that we don't listen on,
 	// because we currently don't have any data for them.
@@ -85,11 +94,11 @@ struct Stream {
 
 	// Queued data, if any. Protected by <queued_data_mutex>.
 	// The data pointers in the iovec are owned by us.
-	std::vector<iovec> queued_data;
-
-	// Index of the last element in queued_data that is suitable to start streaming at.
-	// -1 if none. Protected by <queued_data_mutex>.
-	int queued_data_last_starting_point;
+	struct DataElement {
+		iovec data;
+		StreamStartSuitability suitable_for_stream_start;
+	};
+	std::vector<DataElement> queued_data;
 
 	// Put client to sleep, since there is no more data for it; we will on
 	// longer listen on POLLOUT until we get more data. Also, it will be put
@@ -109,8 +118,14 @@ private:
 
 	// Adds data directly to the stream file descriptor, without adding headers or
 	// going through <queued_data>.
+	// You should hold the owning Server's <mutex>, and probably call
+	// remove_obsolete_starting_points() afterwards.
+	void add_data_raw(const std::vector<DataElement> &data);
+
+	// Remove points from <suitable_starting_points> that are no longer
+	// in the backlog.
 	// You should hold the owning Server's <mutex>.
-	void add_data_raw(const std::vector<iovec> &data);
+	void remove_obsolete_starting_points();
 };
 
 #endif  // !defined(_STREAM_H)
