@@ -16,8 +16,7 @@ class MarkPool;
 
 class UDPStream {
 public:
-	// <mark_pool> can be NULL. Does not take ownership of the mark pool.
-	UDPStream(const sockaddr_in6 &dst, MarkPool *mark_pool, uint32_t pacing_rate);
+	UDPStream(const sockaddr_in6 &dst, uint32_t pacing_rate, int ttl, int multicast_iface_index);
 	~UDPStream();
 
 	void send(const char *data, size_t bytes);
@@ -27,9 +26,6 @@ private:
 
 	sockaddr_in6 dst;
 	int sock;
-	MarkPool *mark_pool;
-	int fwmark;
-	uint32_t pacing_rate;
 };
 
 #endif  // !defined(_UDPSTREAM_H)
